<?php

/**
 * @file theme_registry.alter.inc
 * Alters the theme registry.
 */
// Calls hex_file_scan()
include_once HEX_INCLUDES . '/build_files.inc';

/**
 * Anything containing a theme function or variable process function must be
 * included here in order for it to be discovered.
 */
foreach (hex_get_theme_registry_files() as $file) {
  switch ($file->type) {
    case 'theme':
    case 'func' :
      include_once DRUPAL_ROOT . '/' . $file->uri;
  }
}

/**
 * implements hook_theme_registry_alter()
 */
function hex_theme_registry_hex_alter(&$registry) {

  $active_theme   = hex_active_theme();
  $active_plugins = hex_plugin_list();
  $vars_processors_inc = hex_path('theme', 'hex') . '/includes/vars_processors.inc';

  // hook_theme() for plug-ins.
  foreach ($active_plugins as $plugin) {
    _theme_process_registry($registry, $plugin, 'plugin', $plugin, hex_path('plugin', $plugin));
  }

  // Fill in gaps in the theme registry.
  hex_theme_registry_fill($registry);

  $includes = array();
  foreach (hex_get_theme_registry_files() as $file) {
    switch ($file->type) {
      case 'theme':
      case 'func' :
      case 'vars' :
        $includes[$file->name][end($file->hosts)] = $file->uri;
    }
  }

  foreach ($registry as $theme_hook => &$hook_info) {

    // Get hook info.
    $hex_info = hex_hook_info($registry, $theme_hook);

    // Initialize. Make sure these elements are available.
    // Empty elements will be removed later.
    $hook_includes = &$hook_info['includes'];
    $vars_includes = &$hook_info['vars includes'];
    $preprocesses  = &$hook_info['preprocess functions'];
    $processes     = &$hook_info['process functions'];

    foreach (array('includes', 'vars includes', 'preprocess functions', 'process functions') as $key) {
      if (!isset($hook_info[$key])) {
        $hook_info[$key] = array();
      }
    }

    // Add HOOK_ORIGIN.theme.inc files into Core's ability to include them on
    // the fly through theme() calls. The hook origin is which ever module,
    // theme or plug-in that originally implements the theming hook through
    // hook_theme().
    //
    // Process:
    // - [ MODULE | THEME | PLUGIN ].theme.inc
    if ($hex_info->is_base) {
      foreach ($hex_info->processed_by as $component) {
        if (isset($includes["{$hex_info->origin}.theme"][$component])) {
          $hook_includes[] = $includes["{$hex_info->origin}.theme"][$component];
        }
      }
    }

    // Include:
    // - [ HOOK ].func.inc
    $hook_name = str_replace('_', '-', $theme_hook);

    if ($hex_info->host_type === 'theme'
    &&  $hex_info->implement_type  === 'func'
    &&  isset($includes["$hook_name.func"])) {
      // Use last element which will be in the child-most theme.
      $hook_includes[] = end($includes["$hook_name.func"]);
    }

    // The following files will host variables specific to the theming hook
    // whether it is implemented as a function or template. Some are more
    // focused than others.
    //
    // Process:
    // - [ HOOK ].vars.inc
    // - [ HOOK ].[ ACTIVE-THEME ].vars.inc
    // - [ HOOK ].[ func | tpl ].vars.inc
    // - [ HOOK ].[ ACTIVE-THEME ].[ func | tpl ].vars.inc
    // - [ HOOK--PATTERN ].vars.inc
    // - [ HOOK--PATTERN ].[ ACTIVE-THEME ].vars.inc
    // - [ HOOK--PATTERN ].[ func | tpl ].vars.inc
    // - [ HOOK--PATTERN ].[ ACTIVE-THEME ].[ func | tpl ].vars.inc

    foreach ($hex_info->hooks as $hook) {
      $hook_name = str_replace('_', '-', $hook);

      foreach (array($hook_name,
                    "$hook_name.$active_theme",
                    "$hook_name.{$hex_info->implement_type}",
                    "$hook_name.$active_theme.{$hex_info->implement_type}") as $named) {

        if (isset($includes["$named.vars"])) {

          // Include the file that handles hook.vars.inc files.
          if (!in_array($vars_processors_inc, $hook_includes)) {
            $hook_includes[] = $vars_processors_inc;
          }

          // Move to the front of second phase process so it always comes after
          // preprocess functions but before process functions.
          if (!in_array('hex_variable_include_process', $processes)) {
            array_unshift($processes, 'hex_variable_include_process');
          }

          // Add path to allow the custom preprocessor to load the file.
          $vars_includes = array_merge($vars_includes, array_values($includes["$named.vars"]));
        }
      }
    }

    // Add in variable process functions for plugins.
    // Core only knows of modules and themes so plug-ins are added here manually.
    foreach (array('preprocess', 'process') as $phase) {
      foreach ($active_plugins as $prefix) {

        $var_func = "{$prefix}_$phase";
        if ($hex_info->is_base
        &&  isset($hook_info['template'])
        &&  function_exists($var_func)
        && !in_array($var_func, $hook_info["$phase functions"])) {
          // PLUGIN_preprocess/PLUGIN_process() pushed up front.
          // Must be a base hook and be implemented as a template.
          array_unshift($hook_info["$phase functions"], $var_func);
        }

        $var_func .= "_$theme_hook";
        if (function_exists($var_func)
        && !in_array($var_func, $hook_info["$phase functions"])) {
          // PLUGIN_preprocess_HOOK/PLUGIN_process_HOOK() come last.
          $hook_info["$phase functions"][] = $var_func;
        }
      }
    }

    // This is used to invoke variable processors when hook suggestions are
    // involved. It works on two levels:
    //
    // 1. When theme('hook__suggestion') is invoked, the variable processors
    //    for that hook will not be complete. What this does is add a specialzed
    //    processor (hex_variable_preprocess_base & hex_variable_process_base)
    //    to handle the invocation of the base functions. It does this by looking
    //    at the differences between hook__suggestion and the base hook effectively
    //    providing a complete set of variable processors.
    // 2. When the suggestion is set from a variable processor, it prevents
    //    the hook suggestion from having its own variable processors from
    //    being invoked since it's in the same scope. What this does is add a
    //    specialized processor (hex_variable_preprocess_suggestion &
    //    hex_variable_process_suggestion) which handles the invocation of the
    //    remaining processors for the suggestion.

    foreach (array('preprocess', 'process') as $phase) {
      foreach ($hex_info->suggestions as $suggestion) {
        $suggestion_info = &$registry[$suggestion];

        if (!isset($suggestion_info["$phase functions"])) {
          $suggestion_info["$phase functions"] = array();
        }
        elseif ($hex_info->is_base && !in_array("hex_variable_{$phase}_suggestion", $hook_info["$phase functions"])) {
          $hook_info["$phase functions"][] = "hex_variable_{$phase}_suggestion";
        }

        $ignore = $suggestion_info["$phase functions"];
        $ignore[] = "hex_variable_{$phase}_suggestion";
        $ignore[] = 'hex_variable_include_process';
        foreach ($hook_info["$phase functions"] as $i => $phase_function) {
          if (!in_array($phase_function, $ignore)) {
            $suggestion_info["base $phase functions"][] = $phase_function;
          }
        }

        if (!empty($suggestion_info["base $phase functions"])) {
          if (!isset($suggestion_info['includes'])) {
            $suggestion_info['includes'] = array();
          }
          if (!in_array($vars_processors_inc, $suggestion_info['includes'])) {
            $suggestion_info['includes'][] = $vars_processors_inc;
          }
          if (!in_array("hex_variable_{$phase}_base", $suggestion_info["$phase functions"])) {
            array_unshift($suggestion_info["$phase functions"], "hex_variable_{$phase}_base");
          }
        }
      }
    }

    // Inherit includes from base hook for patterns.
    if (!$hex_info->is_base
    &&  isset($registry[$hex_info->base_hook]['includes'])
    &&  $diff = array_diff($registry[$hex_info->base_hook]['includes'], $hook_includes)) {
      $hook_includes = array_merge($diff, $hook_includes);
    }

    // Optimize the registry.
    foreach (array('includes', 'vars includes', 'preprocess functions', 'process functions') as $key) {
      if (empty($hook_info[$key])) {
        unset($hook_info[$key]);
      }
    }
    // Base hook not needed. This is poorly handed in core's theme function so remove it.
    if (isset($hook_info['base hook'])) {
      unset($hook_info['base hook']);
    }
  }
}

/**
 * Builds metadata about a theming hook for the theme registry.
 */
function hex_hook_info($registry, $theme_hook) {

  $host_types = array(
    'module'            => 'module',
    'base_theme_engine' => 'theme',
    'base_theme'        => 'theme',
    'theme_engine'      => 'theme',
    'theme'             => 'theme',
    'plugin'            => 'plugin',
  );

  $hook_info = $registry[$theme_hook];

  $hex_info = new stdClass();
  $hex_info->host_type      = $host_types[$hook_info['type']];
  $hex_info->implement_type = isset($hook_info['function']) ? 'func' : 'tpl';

  $hook_origins = hex_get_theme_hook_origins();
  $base_hook    = hex_find_parent_hook($hook_origins, $theme_hook, FALSE);

  $hex_info->is_base   = isset($hook_origins[$theme_hook]);
  $hex_info->base_hook = $base_hook;

  // Multiple hooks used since this looks through hook__suggestions.
  $hex_info->hooks = array($theme_hook);
  $hook = $theme_hook;
  // Stop when a base hook is reached or it can't go any further.
  while (!isset($hook_origins[$hook]) && $hook = hex_find_parent_hook($registry, $hook, TRUE)) {
    array_unshift($hex_info->hooks, $hook);
  }

  // Identify the origin of the hook.
  if (isset($hook_origins[$theme_hook])) {
    $hex_info->origin = $hook_origins[$theme_hook];
  }
  elseif ($base_hook && isset($hook_origins[$base_hook])) {
    $hex_info->origin = $hook_origins[$base_hook];
  }

  // Look for the current host.
  if ($hex_info->host_type === 'module') {
    $hex_info->host = $hex_info->origin;
  }
  else {
    foreach (hex_list($hex_info->host_type) as $name) {
      if ($hook_info['theme path'] === hex_path($hex_info->host_type, $name)) {
        $hex_info->host = $name;
        break;
      }
    }
  }

  // Find out which themes or plugins processes the variables.
  $components = hex_list('theme') + hex_list('plugin');
  $hex_info->processed_by = array();

  foreach (array('preprocess', 'process') as $phase) {
    $phase_aggregate = array();
    if (isset($hook_info["$phase functions"])) {
      $phase_aggregate = $hook_info["$phase functions"];
    }
    if ($base_hook && isset($registry[$base_hook]["$phase functions"])) {
      $phase_aggregate = array_merge($phase_aggregate, $registry[$base_hook]["$phase functions"]);
    }
    foreach ($phase_aggregate as $phase_function) {
      $component = substr($phase_function, 0, strpos($phase_function, "_$phase"));
      if (isset($components[$component]) && !in_array($component, $hex_info->processed_by)) {
        $hex_info->processed_by[] = $component;
      }
    }
  }

  // Find "hook suggestions".
  $hex_info->suggestions = array();
  if (isset($hook_origins[$theme_hook])) {
    foreach (preg_grep("/^($theme_hook)__/", array_keys($registry)) as $suggestion) {
      if (!isset($hook_origins[$suggestion])) {
        $hex_info->suggestions[] = $suggestion;
      }
    }
  }

  return $hex_info;
}

/**
 * Fills missing gaps in the theme registry.
 * 
 * Provides the following:
 *  - New hooks based on variable process functions.
 *  - New hooks based on .vars.inc files.
 *  - Pattern based preprocess and process function connected to the new hook.
 */
function hex_theme_registry_fill(&$registry) {

  $hook_origins = hex_get_theme_hook_origins();

  // Register hooks based on variable process functions and .vars.inc files.
  // All user defined functions.
  list(, $user_func) = array_values(get_defined_functions());

  // All known base hooks. Discovered functions must be based on these.
  $hooks = implode('|', array_keys($hook_origins));

  // Find new hook suggestions derived from variable processor names.
  // Create a new entry by inheriting all the data from the closest parent hook.
  foreach (hex_list('theme') + hex_list('plugin') as $prefix) {
    foreach (preg_grep("/^{$prefix}_(pre)?process_($hooks)__/", $user_func) as $processor) {
      $hook = substr($processor, strpos($processor, 'process_') + strlen('process_'));
      if ($parent_hook = hex_find_parent_hook($registry, $hook, TRUE)) {
        // Uses the immediate parent hook. Not necessarily the base hook.
        $registry[$hook] = $registry[$parent_hook];
        // Remove variable processors. They are handled separately.
        foreach (array('preprocess functions', 'process functions') as $phase_process) {
          if (isset($registry[$hook][$phase_process])) {
            unset($registry[$hook][$phase_process]);
          }
        }
        // Add the variable processor.
        $phase = strpos($processor, 'preprocess') ? 'preprocess functions' : 'process functions';
        $registry[$hook][$phase][] = $processor;
      }
    }
  }

  // Find hooks derived from vars.inc files. Same idea as above.
  foreach (hex_get_theme_registry_files() as $file) {
    if ($file->type === 'vars') {
      $hook = str_replace('-', '_', $file->basename);
      if (!isset($registry[$hook]) && $parent_hook = hex_find_parent_hook($registry, $hook, TRUE)) {
        $registry[$hook] = $registry[$parent_hook];
        // Remove variable processors. They are handled separately.
        foreach (array('preprocess functions', 'process functions') as $phase_process) {
          if (isset($registry[$hook][$phase_process])) {
            unset($registry[$hook][$phase_process]);
          }
        }
      }
    }
  }
}

/**
 * Scans for files needed for the theme registry.
 */
function hex_get_theme_registry_files() {
  $cache = &drupal_static(__FUNCTION__, NULL);

  if (!isset($cache)) {
    $cache = array();

    // Start by creating a filter so only valid files are returned.
    $hooks = $origins = array();
    foreach (hex_get_theme_hook_origins() as $hook => $origin) {
      $hooks[] = $hook;
      if (!in_array($origin, $origins)) {
        $origins[] = $origin;
      }
    }

    $origin_regex = '/^(' . implode('|', $origins) . ')$/';
    $hook_regex   = '/^(' . str_replace('_', '-', implode('|', $hooks)) . ')(--[[:alnum:]-]+)?$/';

    foreach (hex_file_scan('/^[[:alpha:]][[:alnum:]._-]+\.(theme|func|vars)\.inc$/') as $files_data) {
      foreach ($files_data as $file_data) {
        $parts = explode('.', $file_data->name);
        $name  = array_shift($parts);
        $type  = array_pop($parts);

        if ($type === 'theme' && preg_match($origin_regex, $name)
        ||  $type === 'func'  && preg_match($hook_regex, $name)
        ||  $type === 'vars'  && preg_match($hook_regex, $name)) {
          $file_data->basename = $name;
          $file_data->type     = $type;
          $cache[] = $file_data;
        }
      }
    }
  }

  return $cache;
}

/**
 * Returns all theming hooks and their origins.
 */
function hex_get_theme_hook_origins() {
  $cache = &drupal_static(__FUNCTION__, NULL);

  if (!isset($cache)) {
    $cache = array();

    // False pass of hook_theme() so the theming hooks implemented by each
    // component can be determined. $theme_hooks does nothing. This is only
    // to prevent potential notices or errors.

    $theme_hooks = array();
    foreach (array('module' => hex_list('module'),
                   'theme'  => hex_list('theme'),
                   'plugin' => hex_list('plugin')) as $type => $origins) {

      foreach ($origins as $origin_key) {
        $hook_theme = $origin_key . '_theme';
        if (function_exists($hook_theme)) {
          $result = $hook_theme($theme_hooks, $type, $origin_key, hex_path($type, $origin_key));
          foreach (array_keys($result) as $hook) {
            // Maintain first origin.
            if (!isset($cache[$hook])) {
              $cache[$hook] = $origin_key;
            }
          }
          $theme_hooks += $result;
        }
      }
    }
  }

  return $cache;
}

/**
 * Looks for the parent hook when given a hook based on a pattern.
 * 
 * @param $hooks
 *  The set of hooks to check against. Either a set of base hooks or the
 *  complete registry. Only the keys are used.
 * @param $hook_pattern
 *  The hook pattern being processed.
 * @param $force_fallback
 *  If set to TRUE, $hook_pattern will not be checked against $hooks before
 *  attempting to find a parent hook. It is important to set this to FALSE
 *  when checking for the base hook since hook patterns defined from
 *  hook_theme() does not need a fallback.
 */
function hex_find_parent_hook($hooks, $hook_pattern, $force_fallback) {
  $hook = $hook_pattern;

  if (!$force_fallback && isset($hooks[$hook])) {
    return FALSE;
  }

  while ($pos = strrpos($hook, '__')) {
    $hook = substr($hook, 0, $pos);
    if (isset($hooks[$hook])) {
      break;
    }
  }

  return $hook !== $hook_pattern ? $hook : FALSE;
}
