<?php

/**
 * Returns the path to the most relevant file by passing in its name.
 *
 * The file returned will be closest to the active theme. This is important to
 * understand when the active theme has a parent or multiple parents and/or
 * plug-ins. It will fallback in this order:
 *
 * - active theme
 * - plug-in hosted by the active theme
 * - parent theme
 * - plug-in hosted by the parent theme
 * - etc...
 *
 * This also depends on the namespace and scope of the defined file extensions
 * and patterns and how this function is called. Namespaces are defined from
 * .info or .pinfo files when defining file extensions and patterns.
 *
 *  file.extensions[NAMESPACE] = .ext
 *
 *  file.patterns[NAMESPACE]   = /^regex$/
 *
 * When a namespace is not defined, it is in the virtual root.
 *
 *  file.extensions[] = .css
 *  file.extensions[] = .js
 *
 *  file.patterns[]   = /^regex$/
 *
 * The scope defines where the search occurs and it must be prefixed before
 * the namespace separated with a colon. Multiple scopes can be set delimited
 * by a vertical bar. Valid values are "all", "theme", "plugin", a specific
 * theme key or plugin key.
 *
 *  file.extensions[SCOPE:NAMESPACE]          = .ext
 *
 *  file.patterns[SCOPE:NAMESPACE]            = /^regex$/
 *  file.patterns[SCOPE1|SCOPE2:NAMESPACE]    = /^regex$/
 *  file.patterns[theme|plugins:color_styles] = /^[[:alpha:]][[:alnum:]._-]+\.colors\.css$/
 *
 * If the scope and namespace is not defined, the scope is automatically set
 * to "theme" or "plugin" and falls into root depending on where it was set.
 * Set from the .pinfo file (plug-in) will set it to "plugin" for example.
 *
 * File patterns can also be set from a hook supplied by Hexagon. The same
 * rules apply as setting them from info files except that it only works with
 * regex patterns:
 *
 *  function [THEME_NAME|PLUGIN_NAME]_search_patterns($existing) {
 *    return array(
 *      'SCOPE:NAMESPACE' => '/^regex$/',
 *    );
 *  }
 *
 * Note: If a file or directory is prefixed with a ~tilde, it will be ignored.
 * This may be useful when you need to disable the file or group of files
 * temporarily. Also keep in mind that the path is cached. The cache monitors
 * the theme registry so if it is cleared, so will the files cache. There's
 * a theme setting to force all theme related caches to clear. See the theme
 * settings page or add this into the active theme's .info file.
 *
 *  settings[development_mode] = true
 *  settings[development_mode:disable_theme_caching] = true
 *
 * @param $namespace
 *  The file name including the namespace prefix.
 * @param $scope
 *  Can be 'all', 'theme' or 'plugin'. It can also be a specific theme or
 *  plug-in name, whichever owns or "hosts" the file. File discovery depends
 *  on how the scope was defined from the .info/.pinfo file. When this
 *  function is called, it can be any sub-set of the defined scope. It can
 *  also be a super-set but the returned file paths will always be limited by
 *  the scope set from .info/.pinfo.
 */
function hex_file_path($namespace, $scope = 'all') {
  $files = hex_cache('files', 'hex')->data;
  return isset($files[$scope][$namespace]) ? end($files[$scope][$namespace]) : NULL;
}

/**
 * Returns an array of file paths for a given name.
 *
 * @see hex_file_path() for more info.
 *
 * @param $namespace
 *  The file name including the namespace prefix.
 * @param $scope
 *  Can be 'all', 'theme' or 'plugin'. It can also be a specific theme or
 *  plug-in name, whichever owns or "hosts" the file. File discovery depends
 *  on how the scope was defined from the .info/.pinfo file. When this
 *  function is called, it can be any sub-set of the defined scope. It can
 *  also be a super-set but the returned file paths will always be limited by
 *  the scope set from .info/.pinfo.
 */
function hex_file_path_multiple($namespace, $scope = 'all') {
  $files = hex_cache('files', 'hex')->data;
  return !empty($files[$scope][$namespace]) ? $files[$scope][$namespace] : array();
}
