<?php

define('HEX_PREVIOUS',    -1);
define('HEX_CURRENT_PAGE', 0);
define('HEX_THEMEAPI',     1);
define('HEX_ACTIVE',       2);
define('HEX_ALL_ACTIVE',   3);
define('HEX_CASCADE',      4);

/**
 * Returns the current theme. Optionally sets the context.
 *
 * @param $context
 *  This should be left blank under most circumstances. Calling this function
 *  with this parameter and $context_hold may affect the returned theme in
 *  subsequent runs. This parameter will stack.
 *  - HEX_ACTIVE (default)
 *    Return the active theme.
 *  - HEX_THEMEAPI
 *    Return the theme associated with the form in the theme configuration
 *    page. It is only relevant within the scope of the following hooks:
 *    - hook_form_system_theme_settings_alter()
 *    - hook_form_system_theme_settings_hex_alter()
 *    - hook_form_theme_settings_hex_alter()
 *  - HEX_CURRENT_PAGE
 *    Return the theme used to render the page. This is only useful when a
 *    separate administration theme is active while in the theme settings
 *    form. Hexagon base uses this information to prevent the theme from fully
 *    loading when the admin theme is not connected to Hexagon.
 *  - HEX_PREVIOUS
 *    This will return the previous context in the stack.
 * @param $context_hold
 *  Holds the context for subsequent runs.
 */
function hex_active_theme($context = NULL, $context_hold = FALSE) {
  static $stack = array(HEX_ACTIVE);

  $contexts = array(
    HEX_ACTIVE       => $GLOBALS['theme_static'],
    HEX_THEMEAPI     => $GLOBALS['theme_key'],
    HEX_CURRENT_PAGE => $GLOBALS['theme'],
  );

  if (!isset($context)) {
    return $contexts[$stack[0]];
  }
  elseif (isset($contexts[$context]) && !$context_hold) {
    return $contexts[$context];
  }

  if (isset($contexts[$context]) && $context_hold) {
    array_unshift($stack, $context);
  }
  elseif ($context === HEX_PREVIOUS && count($stack) > 1) {
    array_shift($stack);
  }
  // This global used to target caching.
  // Faster than constantly calling hex_active_theme().
  $GLOBALS['theme_target'] = $contexts[$stack[0]];

  return $contexts[$stack[0]];
}

/**
 * List the active themes starting from parents.
 *
 * @see hex_active_theme()
 */
function hex_active_themes($context = NULL) {
  static $theme_lists;

  $theme = hex_active_theme($context);
  if (!isset($theme_lists[$theme])) {
    $themes = list_themes();

    $theme_lists[$theme] = array();
    $_theme = $theme;
    while (isset($_theme)) {
      $theme_lists[$theme][$_theme] = $_theme;
      $_theme = isset($themes[$_theme]->base_theme) ? $themes[$_theme]->base_theme : NULL;
    }
    $theme_lists[$theme] = array_reverse($theme_lists[$theme]);
  }

  return $theme_lists[$theme];
}

/**
 * Get the values defined within the .info file.
 *
 * @param $info_key
 *  The key to retrieve.
 * @param $default
 *  Fall back value if nothing is found.
 * @param $theme
 *  Can be a theme key which will restricted it to a specific theme. Other
 *  values allowed are HEX_ACTIVE (default) or HEX_THEMEAPI.
 */
function hex_theme_info($info_key, $default = NULL, $theme = HEX_ACTIVE) {

  if ($theme === HEX_ACTIVE || $theme === HEX_THEMEAPI) {
    $theme = hex_active_theme($theme);
  }
  $list_themes = list_themes();

  return isset($list_themes[$theme]->info[$info_key]) ? $list_themes[$theme]->info[$info_key] : $default;
}

/**
 * Returns meta information for a given theme hook.
 * 
 * @param $hook
 *  The hook to target.
 * @param $info
 *  Optional sub-key for the hook. If not set, it will return all the meta
 *  info available.
 */
function hex_theme_hook_info($hook, $info = NULL) {
  static $hooks;

  if (!isset($hooks)) {
    $hooks = theme_get_registry();
  }

  if ($info) {
    return isset($hooks[$hook][$info]) ? $hooks[$hook][$info] : NULL;
  }
  else {
    return isset($hooks[$hook]) ? $hooks[$hook] : NULL;
  }
}

/**
 * A more flexible version of core's theme_get_setting().
 *
 * The setting is obtained from the first value found in the following sources:
 * - saved values from the theme settings form
 * - saved values from the global theme settings form
 * - default theme-specific settings defined in the theme's .info file
 *
 * The list above will cycle through each connected base and sub-themes with
 * sub-themes having priority for their values when the $theme parameter is
 * set to HEX_CASCADE (default).
 *
 * Core's theme_get_setting() is designed to fall back automatically but it's
 * currently broken.
 *
 * @see http://api.drupal.org/api/search/7/theme_get_setting
 * @see http://drupal.org/node/761608
 *  core issue: inconsistent theme data
 *
 * @param $name
 *  The name of the setting to be retrieved. It can be a plain string, named
 *  hierarchy or parent conditional.
 *  - named hierarchy:
 *    This is similar to the DNS namespace in terms of hierarchy but strung
 *    together in reverse. The left most element is the root of the setting
 *    which will return an array of settings. You can traverse any child
 *    setting with the dot to get more specific data, e.g., 'parent.child.etc'.
 *  - parent conditional:
 *    This depends on a parent value before revealing its own value. Assuming
 *    the hypothetical 'power' setting has a value, 'power:lights' will return
 *    its own value. Without the parent, 'power:lights' will always return
 *    NULL. A working example of this is the 'development_mode' setting which
 *    is a master switch for many others.
 * @param $default
 *  The default value if nothing was found.
 * @param $theme
 *  Can be a theme key which will restricted it to a specific theme. Other
 *  values allowed are HEX_CASCADE (default), HEX_ACTIVE or HEX_THEMEAPI.
 */
function hex_setting($name = NULL, $default = NULL, $theme = HEX_CASCADE) {
  $cache = hex_cache('settings', 'hex')->data;

  if ($theme === HEX_ACTIVE || $theme === HEX_THEMEAPI) {
    $theme = hex_active_theme($theme);
  }
  elseif ($theme == HEX_CASCADE && !empty($cache->cascade[$name])) {
    $theme = $cache->cascade[$name];
  }

  return isset($cache->settings[$theme][$name]) ? $cache->settings[$theme][$name] : $default;
}

/**
 * Retrieve multiple settings from multiple sources or "scopes".
 *
 * Example:
 *
 *  $settings = hex_settings(array(
 *    SETTING_KEY => SCOPE,
 *    'key_1'     => HEX_ACTIVE,
 *    'key_2'     => HEX_ALL_ACTIVE,
 *    'key_3'     => HEX_CASCADE,
 *    'key_4'     => THEMENAME,
 *  ));
 *
 * TODO: document this further.
 *
 * @param $setting_keys
 *  Array of setting keys and their scope.
 */
function hex_settings($setting_keys) {
  $cache = hex_cache('settings', 'hex')->data;

  foreach ($setting_keys as $setting_key => $scope) {
    $settings[$setting_key] = $scopes = array();

    if ($scope == HEX_ALL_ACTIVE) {
      $scopes = hex_active_themes();
    }
    elseif ($scope == HEX_ACTIVE) {
      $scopes[] = hex_active_theme();
    }
    elseif ($scope == HEX_CASCADE && isset($cache->cascade[$setting_key])) {
      $scopes[] = $cache->cascade[$setting_key];
    }
    elseif (in_array($scope, hex_active_themes())) {
      $scopes[] = $scope;
    }

    foreach ($scopes as $scope) {
      if (isset($cache->settings[$scope][$setting_key])) {
        $settings[$setting_key][] = array('setting' => $cache->settings[$scope][$setting_key],
                                          'scope'   => $scope);
      }
    }
  }

  return $settings;
}

/**
 * Pass in a type to get a list of active components. This function is useful
 * when accessing multiple data types in single way. This will always return a
 * flat array.
 *
 * @param $type
 *  Possible values: 'theme', 'plugin' or 'module'.
 */
function hex_list($type) {
  switch ($type) {
    case 'theme'  : return hex_active_themes();
    case 'plugin' : return hex_plugin_list('all.active');
    case 'module' : return module_list();
  }
}

/**
 * Provides a standard way for retrieving paths to themes, plugins, etc...
 *
 * @see drupal_get_path()
 *
 * @param $type
 *  Anything accepted by drupal_get_path() in addition to 'plugin'.
 * @param $name
 *  Name of the item to retrieve.
 */
function hex_path($type, $name) {
  switch ($type) {
    case 'plugin': return hex_plugin_path($name);
    default:       return drupal_get_path($type, $name);
  }
}

/**
 * Provides status and logs them for theme developers.
 *
 * Users granted 'administer themes' permission will be shown a status
 * message. Notices should be minimal and be shown only to relevant roles.
 *
 * When the $type is 'debug', the theme setting 'development_mode' and
 * 'development_mode:debug_log' must be enabled for the active theme. Debug
 * entries will not be shown on the front-end. They will go straight to the
 * watchdog log.
 *
 * @see drupal_set_message()
 * @see watchdog()
 *
 * @param $message
 *  The message text. Works the same way as watchdog() but it is also used
 *  for the status text.
 * @param $variables
 *  Placeholder variables for $messages.
 * @param $type
 *  The level of the message. Can only be one of four possible values:
 *  error, warning, status or debug.
 */
function hex_notify($message, $variables = array(), $type = 'status') {
  static $status;

  if (!isset($status)) {
    $status = array();
    if (user_access('administer themes')) {
      $status = array('error'   => WATCHDOG_ERROR,
                      'warning' => WATCHDOG_WARNING,
                      'status'  => WATCHDOG_INFO);
    }
    if (hex_setting('development_mode:debug_log', 0, HEX_ACTIVE)) {
      $status['debug'] = WATCHDOG_DEBUG;
    }
  }

  if (isset($status[$type])) {
    if ($type !== 'debug') {
      drupal_set_message(t($message, $variables), $type, FALSE);
    }
    watchdog('hexagon', $message, $variables, $status[$type]);
  }
}
