<?php

// Used to flag processed suggestions.
define('HEX_SUGGESTION_PROCESSED', 100);

/**
 * When any of the following files are detected for a specific hook, they will
 * be included and processed:
 *
 * - [ HOOK ].vars.inc
 * - [ HOOK ].[ ACTIVE-THEME ].vars.inc
 * - [ HOOK ].[ func | tpl ].vars.inc
 * - [ HOOK ].[ ACTIVE-THEME ].[ func | tpl ].vars.inc
 * - [ HOOK--PATTERN ].vars.inc
 * - [ HOOK--PATTERN ].[ ACTIVE-THEME ].vars.inc
 * - [ HOOK--PATTERN ].[ func | tpl ].vars.inc
 * - [ HOOK--PATTERN ].[ ACTIVE-THEME ].[ func | tpl ].vars.inc
 */
function hex_variable_include_process(&$variables, $hook) {
  $hook_target = $hook;
  // Check for hook suggestion.
  if ($suggestion = hex_find_theme_hook_suggestion($variables)) {
    $hook_target = $suggestion;
  }

  if ($variable_includes = hex_theme_hook_info($hook_target, 'vars includes')) {
    // Allow both common spellings.
    $vars = &$variables;
    // Extract the variables to a local namespace. This is not expensive.
    extract($variables, EXTR_REFS + EXTR_SKIP);
    // Include files.
    foreach ($variable_includes as $variable_include) {
      include DRUPAL_ROOT . '/' . $variable_include;
    }
  }
}

/**
 * This is only used for base hooks and a suggestion is made by other variable
 * processors. Setting a suggestion prevents the processors connected to the
 * suggestion from running. This effectively fills the gap by allowing the
 * processors specific to the suggestion to run.
 *
 * @see hex_suggestion_processors()
 */

function hex_variable_preprocess_suggestion(&$variables, $hook) {
  static $processors;

  if ($suggestion = hex_find_theme_hook_suggestion($variables)) {
    if (!isset($processors[$suggestion])) {
      $processors[$suggestion] = hex_suggestion_processors($hook, $suggestion, 'preprocess');
    }
    foreach ($processors[$suggestion] as $processor) {
      $processor($variables, $suggestion);
    }
  }
}

/**
 * This is only used for base hooks and a suggestion is made by other variable
 * processors. Setting a suggestion prevents the processors connected to the
 * suggestion from running. This effectively fills the gap by allowing the
 * processors specific to the suggestion to run.
 *
 * @see hex_suggestion_processors()
 */
function hex_variable_process_suggestion(&$variables, $hook) {
  static $processors;

  if ($suggestion = hex_find_theme_hook_suggestion($variables)) {
    if (!isset($processors[$suggestion])) {
      $processors[$suggestion] = hex_suggestion_processors($hook, $suggestion, 'process');
    }
    foreach ($processors[$suggestion] as $processor) {
      $processor($variables, $suggestion);
    }
  }
}

/**
 * When variable processors for a hook suggestion is detected, it will return
 * the remaining processor functions. The caller will fill in the gaps
 * effectively running the full set of processors.
 *
 * @see hex_variable_preprocess_suggestion()
 * @see hex_variable_process_suggestion()
 */
function hex_suggestion_processors($hook, $suggestion, $phase) {
  $processor_list = array();

  if ($hook !== $suggestion) {
    foreach ((array) hex_theme_hook_info($suggestion, 'includes') as $include_file) {
      include_once DRUPAL_ROOT . '/' . $include_file;
    }

    foreach ((array) hex_theme_hook_info($suggestion, "{$phase} functions") as $processor) {
      if ($processor !== "hex_variable_{$phase}_base" && function_exists($processor)) {
        $processor_list[] = $processor;
      }
    }
  }

  return $processor_list;
}

/**
 * This is only used for pattern based hooks or "hook suggestions" since they
 * do not carry a full set of variable processor functions.
 */
function hex_variable_preprocess_base(&$variables, $hook) {
  foreach (hex_theme_hook_info($hook, "base preprocess functions") as $processor) {
    if (function_exists($processor)) {
      $processor($variables, $hook);
    }
  }
}

/**
 * This is only used for pattern based hooks or "hook suggestions" since they
 * do not carry a full set of variable processor functions.
 */
function hex_variable_process_base(&$variables, $hook) {
  foreach (hex_theme_hook_info($hook, "base process functions") as $processor) {
    if (function_exists($processor)) {
      $processor($variables, $hook);
    }
  }
}

/**
 * Finds a theming hook from suggestions set from variable processors.
 * 
 * This will strip 'theme_hook_suggestion' of any suggestions that are not
 * registered in the theme registry. Two new variables will be introduced,
 * 'available_suggestions' and 'unavailable_suggestions'. The latter is useful
 * for getting context on what suggestions are available that never caught on.
 */
function hex_find_theme_hook_suggestion(&$variables) {
  $suggestions = array();
  if (!empty($variables['theme_hook_suggestions'])) {
    $suggestions = &$variables['theme_hook_suggestions'];
  }
  if (!empty($variables['theme_hook_suggestion'])) {
    $suggestions[] = $variables['theme_hook_suggestion'];
    unset($variables['theme_hook_suggestion']);
  }
  if (!isset($variables['available_suggestions'])) {
    $variables['available_suggestions'] = array();
  }
  if (!isset($variables['unavailable_suggestions'])) {
    $variables['unavailable_suggestions'] = array();
  }
  foreach (array_reverse($suggestions, TRUE) as $i => $suggestion) {
    if ($i >= HEX_SUGGESTION_PROCESSED) {
      continue;
    }
    unset($suggestions[$i]);

    if (hex_theme_hook_info($suggestion)) {
      $suggestions[HEX_SUGGESTION_PROCESSED + $i] = $suggestion;
      $variables['available_suggestions'][$i] = $suggestion;
    }
    else {
      $variables['unavailable_suggestions'][$i] = $suggestion;
    }
  }
  // Return first item.
  return reset($variables['available_suggestions']);
}
