<?php

/**
 * @file template.php
 * Contains the base functionality for Hexagon and includes other files when
 * necessary.
 */
define('HEX_ROOT',     DRUPAL_ROOT . '/' . drupal_get_path('theme', 'hex'));
define('HEX_INCLUDES', HEX_ROOT . '/includes');

_hex_initialize();

/**
 * Initializes the theme.
 */
function _hex_initialize() {
  // Hold the theme_key.
  $GLOBALS['theme_static'] = $GLOBALS['theme_key'];
  $GLOBALS['theme_target'] = $GLOBALS['theme_key'];

  include_once HEX_INCLUDES . '/meta_theme.inc';
  include_once HEX_INCLUDES . '/meta_plugin.inc';
  include_once HEX_INCLUDES . '/meta_files.inc';
  include_once HEX_INCLUDES . '/alter/alter_stubs.inc';

  // Load and initialize the theme and plug-ins only if Hexagon or any of its
  // child themes are active. This conditional loading eliminates any possible
  // conflicts when inside the theme settings form while this theme or any child
  // is not currently active.

  if (!in_array('hex', hex_active_themes(HEX_CURRENT_PAGE))) {
    return;
  }

  if (hex_setting('development_mode:disable_theme_caching', 0, HEX_ACTIVE)) {
    // Clear theme .info data and registry.
    system_rebuild_theme_data();
    drupal_theme_rebuild();
  }
  if (cache_get('theme_registry:' . hex_active_theme(), 'cache') === FALSE) {
    // Follow the registry and clear hex specific caches.
    hex_clear_caches();
    // Include alter files early so various functions can be discovered by core.
    foreach (hex_file_path_multiple('hook_alter/theme_registry.alter.inc') as $file) {
      include_once DRUPAL_ROOT . '/' . $file;
    }
  }

  // Initialize.
  foreach (hex_active_themes() as $theme) {
    if (function_exists($theme . '_initialize')) {
      call_user_func($theme . '_initialize');
    }
    foreach (hex_plugin_list("$theme.active") as $plugin) {
      if (file_exists($file = hex_path('plugin', $plugin) . "/$plugin.plug.inc")) {
        include_once DRUPAL_ROOT . '/' . $file;
      }
      if (function_exists($plugin . '_initialize')) {
        call_user_func($plugin . '_initialize');
      }
    }
  }
}

/**
 * Clears all caches held by Hexagon.
 */
function hex_clear_caches() {
  drupal_static_reset('hex_cache');
  cache_clear_all('hex_cache', 'cache', TRUE);
}

/**
 * Hexagon caching API.
 *
 * Gets the cache for the given $cache_key. If it is not available, the
 * function responsible for building it is called with the returned data saved
 * to Drupal cores caching system.
 *
 * Any theme or plug-in can take advantage of this. In order to do so, it must
 * implement hook_cache_build(). The 'hook' is determined by the $host
 * parameter. It is the theme or plug-in handling the particular $cache_key.
 *
 * @param $cache_key
 *  The key used to store and retrieve the cache.
 * @param $host
 *  The host theme or plug-in responsible for building the cache.
 */
function hex_cache($cache_key, $host) {
  // Use the advanced drupal_static() pattern, since this is called very often.
  static $drupal_static_fast;
  if (!isset($drupal_static_fast)) {
    $drupal_static_fast['caches'] = &drupal_static(__FUNCTION__);
  }
  $cid = $host . '_cache:' . $cache_key . ':' . $GLOBALS['theme_target'];
  $cached = &$drupal_static_fast['caches'][$cid];

  if (!isset($cached) && !$cached = cache_get($cid)) {
    timer_start($cid);

    cache_set($cid, call_user_func($host . '_cache_build', $cache_key));
    $cached = cache_get($cid);

    timer_stop($cid);
    hex_notify('Created and saved cache id !id in !timer milliseconds.',
               array('!id' => $cid, '!timer' => timer_read($cid)),
               'debug');
  }

  return $cached;
}

/**
 * implements hook_cache_build()
 *
 * This includes the file responsible for building the cache and invokes the
 * function that builds it returning the built data. Called from hex_cache()
 * which will save it to Drupal cores caching system.
 */
function hex_cache_build($cache_key) {
  include_once HEX_INCLUDES . "/build_$cache_key.inc";
  return call_user_func("hex_build_$cache_key");
}
